//	Ball.cpp - The ball class.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "drawing/Drawer.h"
#include "GamePlayState.h"
#include "MiscHelper.h"
#include "Ball.h"

//------------------------------------------------------------------------------
Ball::Ball(const TwoFloats& pos):
Collidable(L"Ball"),
position(pos),
speed(1.0f),
size(1.0f),
sizeTweener(size, 1.0f, 0.05f),
flameIndex(0),
flameCount(7),
newCol(1.0f, 0.75f, 0.45f),
oldCol(0.53f, 0.45f, 1.0f),
flameInterp(0.0f)
{
	int i;

	flameImages[0] = L"PlayerFlame";
	flameImages[1] = L"PlayerFlame1";
	flameImages[2] = L"PlayerFlame2";
	flameImages[3] = L"PlayerFlame3";
	flameImages[4] = L"PlayerFlame2";
	flameImages[5] = L"PlayerFlame1";

	flameOffsets[0] = 0.0f;
	flameOffsets[1] = 4.0f;
	flameOffsets[2] = 9.0f;
	flameOffsets[3] = 15.0f;
	flameOffsets[4] = 9.0f;
	flameOffsets[5] = 4.0f;

	sizeTweener.setIndex(1.0f);

	for(i=0;i<TailSize;++i)
	{
		tailPos.push_back(pos);
		tailSize.push_back(fakeLog((float)i/(float)TailSize) * 16.0f);
		tailCol.push_back(1.0f);
		tailAlpha.push_back(fakeLog((float)i/(float)TailSize) * 0.66f);
	}
}

//------------------------------------------------------------------------------
Ball::~Ball()
{

}

//------------------------------------------------------------------------------
void Ball::update()
{
	if(direction != 0.0f)
	{
		position += direction * speed;
		direction *= 0.99f;

		if(direction.getMagnitude() < 0.1f)
			direction = 0.0f;
	}

	tailPos.erase(tailPos.begin());
	tailPos.push_back(position);

	if(!sizeTweener.getFinished())
		sizeTweener.tick();

	if(speed > 2.74f)
	{
		--flameCount;
		if(!flameCount)
		{
			flameCount = 7;
			++flameIndex;
			flameIndex %= 6;
		}

		if(!(flameCount%3))
		{
			particles.setColour(oldCol + ((newCol-oldCol) * flameInterp));
			particles.trigger(position);
		}

		flameInterp += 0.1f;
		if(flameInterp >= 1.0f)
		{
			ThreeFloats tempCol(newCol);

			newCol = oldCol;
			oldCol = tempCol;

			flameInterp = 0.0f;
		}
	}

	particles.update();
}

//------------------------------------------------------------------------------
void Ball::arenaWallCheck(const TwoFloats& screenSize)
{
	const TwoFloats arenaTopLeft(64.0f+12.0f, 64.0f+12.0f);
	const TwoFloats arenaBottomRight(screenSize.x-64.0f-12.0f,
									 screenSize.y-32.0f-12.0f);
	const float goalTop = 64.0f + ((screenSize.y-96.0f)*0.5f)-64.0f + 12.0f;
	const float goalBottom = goalTop + 128.0f - 12.0f;

	//Make sure we stay inside the arena.
	if(position.y < arenaTopLeft.y)
		position.y = arenaTopLeft.y;
	else if(position.y > arenaBottomRight.y)
		position.y = arenaBottomRight.y;

	if((position.y < goalTop) || (position.y > goalBottom))
	{
		if(position.x < arenaTopLeft.x)
			position.x = arenaTopLeft.x;
		else if(position.x > arenaBottomRight.x)
			position.x = arenaBottomRight.x;
	}
	else
	{
		if(position.y <= goalTop)
			position.y = goalTop + 0.5f;
		else if(position.y >= goalBottom)
			position.y = goalBottom - 0.5f;

		if(position.x < (arenaTopLeft.x-40.0f))
			position.x = arenaTopLeft.x-40.0f;
		else if(position.x > (arenaBottomRight.x+40.0f))
			position.x = arenaBottomRight.x+40.0f;
	}
}

//------------------------------------------------------------------------------
void Ball::draw(shared_ptr<Drawer>& d,
				float timestepInterp,
				const ThreeFloats& leftCol,
				const ThreeFloats& rightCol)
{
	const ThreeFloats flameCol(oldCol + ((newCol-oldCol) * flameInterp));
	float interp = (position.x/d->getDrawingAreaSize().x);
	ThreeFloats col(rightCol);
	TwoFloats tempPos;

	col += (leftCol-rightCol) * interp;
	col += 0.15f;

	tailCol.erase(tailCol.begin());
	tailCol.push_back(col);

	d->drawConnectedLine(L"",
						 tailPos,
						 tailSize,
						 tailCol,
						 tailAlpha);

	if(speed > 2.74f) //i.e. 1.0 + (87% of 2.0)
	{
		TwoFloats tempDir(direction.x, -direction.y);
		float angle = tempDir.getAngle() + NIALL_PI;

		d->translate(position);
		d->rotate(angle, 0.0f);

		//Draw flame.
		tempPos.x = -31.0f;
		tempPos.y = -73.0f + flameOffsets[flameIndex];
		d->drawImage(flameImages[flameIndex],
					 tempPos,
					 1.0f,
					 flameCol);

		d->rotate(-angle, 0.0f);
		d->translate(TwoFloats(-position.x, -position.y));
	}

	//Draw particles.
	particles.draw(d, timestepInterp);

	d->drawImage(L"Ball",
				 position-12.0f,
				 size,
				 col);
}

//------------------------------------------------------------------------------
void Ball::reset(const TwoFloats& pos)
{
	unsigned int i;

	position = pos;
	direction = 0.0f;
	speed = 1.0f;

	for(i=0;i<tailPos.size();++i)
		tailPos[i] = position;
}

//------------------------------------------------------------------------------
void Ball::setSlow()
{
	speed = 0.25f;
}

//------------------------------------------------------------------------------
void Ball::kick(const TwoFloats& dir)
{
	float mag;

	direction += dir;

	//Limit speed to 64 pixels.
	mag = direction.getMagnitude();
	if(mag > 64.0f)
		direction /= (mag/64.0f);

	size = 0.66f;
	sizeTweener.reset(1.0f);
}

//------------------------------------------------------------------------------
void Ball::updateSpeed(float val)
{
	speed = 1.0f + (val * 2.0f);
}

//------------------------------------------------------------------------------
bool Ball::interestedInCollisionsWith(const std::wstring& entity) const
{
	return true;
}

//------------------------------------------------------------------------------
bool Ball::collide(const shared_ptr<Collidable>& other, GamePlayState& state)
{
	float pan;
	float level;
	float reflectMag;
	TwoFloats mirrorVector((position-other->getPosition()).normal());

	if(other->getName() != L"TailWall")
	{
		if(other->getSpeed() <= 0.0f)
		{
			reflectMag = direction.getMagnitude();
			direction = direction.normal();
			direction = direction.mirror(mirrorVector).normal();
			direction *= reflectMag;
		}
		else
			direction = mirrorVector * 6.0f;
	}
	else
		direction = mirrorVector * direction.getMagnitude();

	level = direction.getMagnitude();
	if(level > 1.0f)
		level = 1.0f;
	pan = position.x/1280.0f;
	if(pan > 1.0f)
		pan = 1.0f;
	state.triggerSFX(level, pan);

	size = 0.66f;
	sizeTweener.reset(1.0f);

	return true;
}
