//	GamePlayState.h - The main gameplay state.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef GAMEPLAYSTATE_H_
#define GAMEPLAYSTATE_H_

#include "drawing/TextLayout.h"
#include "ParticleGenerator.h"
#include "PickupAnnouncer.h"
#include "GameState.h"
#include "Tweener.h"
#include "VUMeter.h"

#include <vector>

class ScreamingSnakeballGlobalData;
class SnakeballSounder;
class Collidable;
class Player;
class Ball;
class Wall;

///	The main gameplay state.
class GamePlayState : public GameState
{
  public:
	///	Constructor.
	GamePlayState(Application& app, SounderPtr& sound);
	///	Destructor.
	~GamePlayState();

	///	Called frequently to update the state.
	void update();
	///	Called to draw the state.
	void draw(DrawerPtr& d, float timestepInterp);

	///	Called when a key down event is received.
	void keyDown(const std::wstring& action,
				 const std::wstring& keyPressed,
				 int keyModifier);
	///	Called when a key up event is received.
	void keyUp(const std::wstring& action,
			   const std::wstring& keyPressed,
			   int keyModifier);
	///	Called when a joystick axis changes.
	void joyMove(const std::wstring& action,
				 float value,
				 int joystick,
				 int axis);

	///	Triggers a sound effect.
	void triggerSFX(float level, float pan);

	///	Returns the name of this state.
	const std::wstring getName() const {return L"GamePlayState";};
	///	Used to construct instances of this state.
	static GameStateCreator<GamePlayState> creator;
  private:
	///	Helper method. Handles the collision detection.
	void collisionDetection();
	///	Helper method. Ensures everything stays within the arena bounds.
	void arenaWallCheck(shared_ptr<Collidable>& collidable);
	///	Helper method. Tests collision against one of the player's tails.
	void tailCheck(shared_ptr<Collidable>& collidable,
				   const shared_ptr<Player>& player);
	///	Helper method. Triggers background flash, screen shake etc.
	/*!
		\param pos The position of the collision.
	 */
	void triggerCollisionEffects(const TwoFloats& pos);

	///	The current background colour.
	ThreeFloats backgroundColour;
	///	Tweener for the background colour.
	STweener<ThreeFloats> backgroundTweener;
	///	The current border colour.
	ThreeFloats borderColour;
	///	Tweener for the border colour.
	STweener<ThreeFloats> borderTweener;
	///	Used to shake the screen.
	float shake;
	///	Used to shake the screen.
	float shakeIndex;

	///	The line left colour.
	ThreeFloats lineLeft;
	///	Tweener for the line left colour.
	STweener<ThreeFloats> lineLeftTweener;
	///	The line right colour.
	ThreeFloats lineRight;
	///	Tweener for the line right colour.
	STweener<ThreeFloats> lineRightTweener;
	///	The top and bottom lines.
	std::vector<TwoFloats> linePos[2];
	///	The top and bottom lines.
	std::vector<float> lineSize;
	///	The top and bottom lines.
	std::vector<ThreeFloats> lineCol;
	///	The top and bottom lines.
	std::vector<float> lineAlpha;

	///	The number of points scored by the left player.
	int leftScore;
	///	The number of points scored by the right player.
	int rightScore;
	///	TextLayout for the left score.
	TextLayout leftScoreLayout;
	///	TextLayout for the right score.
	TextLayout rightScoreLayout;

	///	Used to calculate remainingTime.
	uint32_t lastTicks;
	///	The remaining time in seconds.
	float remainingTime;
	///	Used to pause before resetting the game after a goal.
	int goalPause;

	///	The red VU meter.
	VUMeter redMeter;
	///	The blue VU meter.
	VUMeter blueMeter;

	///	The red player's power level.
	float redPower;
	///	The blue player's power level.
	float bluePower;

	///	The two players.
	std::vector<shared_ptr<Player> > players;
	///	The ball.
	std::vector<shared_ptr<Ball> > balls;
	///	The game's collidables.
	std::vector<weak_ptr<Collidable> > collidables;

	///	Collidable used to represent the arena walls in collisionDetection().
	shared_ptr<Wall> arenaWall;
	///	Collidable used to represent the snake tails in collisionDetection().
	shared_ptr<Wall> tailWall;

	///	The particles from collisions.
	ParticleGenerator particles;
	///	Used to limit the number of times the background colour can be incremented during a frame.
	int backgroundColLimit;

	///	The announcement text.
	PickupAnnouncer announcer;
	///	The initial announcement strings.
	std::vector<std::wstring> initialAnnouncement;
	///	The colours of the initial announcement strings.
	std::vector<ThreeFloats> initialColours;

	///	True if we're playing Whispering Snakeball.
	bool whispering;
	///	All the available whisper sounds.
	std::vector<std::wstring> whispers;
	///	The last selected whisper.
	int lastWhisper;

	///	True if we're playing Shouting Snakeball.
	bool shouting;

	///	Our sounder.
	shared_ptr<SnakeballSounder> sounder;
	///	Our global data.
	shared_ptr<ScreamingSnakeballGlobalData> globalData;
};

#endif
